/*
 * utime() for VMS, Steve Drew, 1990.
 *
 * This function performs the equivlent function as utime() for unix.
 * It modifies the file headers creation/revised date stamps.
 * Under VMS this takes alot of effort with working with RMS structures.
 *
 * Most of this code taken from FILE by Joe Meadows Jr, at the Fred 
 * Hutchinson Cancer Research Center.
 */
#include <descrip.h>
#include <ssdef.h>
#include <rms.h>
#include <stdio.h>
#include <iodef.h>
#include <atrdef.h>
#include <time.h>
#include "fibdef.h"
#include <starlet.h>
#include <string.h>
#include "fatdef.h"
#include "fchdef.h"
#include "fjndef.h"
 
#define RTYPE fat$r_rtype_overlay.fat$r_rtype_bits
#define RATTRIB fat$r_rattrib_overlay.fat$r_rattrib_bits
#define EFBLK fat$r_efblk_overlay.fat$r_efblk_fields
#define HIBLK fat$r_hiblk_overlay.fat$r_hiblk_fields
#define VAXTIME(a) long a [2]
#define VAXTIME_PTR(a)  long (*a)[]

struct utimebuf {
        long actime;
        long modtime;
};

int bin_date(char *vaxtime_s,VAXTIME_PTR(vaxtime_val));
int unix_to_vmstime(time_t time_l,VAXTIME_PTR(vaxtime_val));

utime(file,utimebuf)
char *file;
struct utimebuf *utimebuf;
{
  static struct FAB Fab;
  static struct NAM Nam;
  static struct fibdef Fib; /* short fib */
 
  static struct dsc$descriptor FibDesc =
    {sizeof(Fib),DSC$K_DTYPE_Z,DSC$K_CLASS_S,(char *)&Fib};
  static struct dsc$descriptor_s DevDesc =
    {0,DSC$K_DTYPE_T,DSC$K_CLASS_S,&Nam.nam$t_dvi[1]};
  static struct fatdef Fat;
  static union {
    struct fchdef fch;
    long int dummy;
  } uchar;
  static struct fjndef jnl;
  static long int Cdate[2],Rdate[2],Edate[2],Bdate[2];
  static short int revisions;
  static unsigned short uic[2];
  static union {
    unsigned short int value;
    struct {
      unsigned system : 4;
      unsigned owner : 4;
      unsigned group : 4;
      unsigned world : 4;
    } bits;
  } prot;
#if defined(__DECC) && defined(VAX) 
  static struct atrdef Atr[] = {
    sizeof(Fat),ATR$C_RECATTR,(unsigned int)&Fat,        /* record attributes */
    sizeof(uchar),ATR$C_UCHAR,(unsigned int)&uchar,      /* File characteristics */
    sizeof(Cdate),ATR$C_CREDATE,(unsigned int)&Cdate[0], /* Creation date */
    sizeof(Rdate),ATR$C_REVDATE,(unsigned int)&Rdate[0], /* Revision date */
    sizeof(Edate),ATR$C_EXPDATE,(unsigned int)&Edate[0], /* Expiration date */
    sizeof(Bdate),ATR$C_BAKDATE,(unsigned int)&Bdate[0], /* Backup date */
    sizeof(revisions),ATR$C_ASCDATES,(unsigned int)&revisions, /* number of revisions */
    sizeof(prot),ATR$C_FPRO,(unsigned int)&prot,         /* file protection  */
    sizeof(uic),ATR$C_UIC,(unsigned int)&uic,            /* file owner */
    sizeof(jnl),ATR$C_JOURNAL,(unsigned int)&jnl,        /* journal flags */
    0,0,0
  } ;
#else
  static struct atrdef Atr[] = {
    sizeof(Fat),ATR$C_RECATTR,&Fat,        /* record attributes */
    sizeof(uchar),ATR$C_UCHAR,&uchar,      /* File characteristics */
    sizeof(Cdate),ATR$C_CREDATE,&Cdate[0], /* Creation date */
    sizeof(Rdate),ATR$C_REVDATE,&Rdate[0], /* Revision date */
    sizeof(Edate),ATR$C_EXPDATE,&Edate[0], /* Expiration date */
    sizeof(Bdate),ATR$C_BAKDATE,&Bdate[0], /* Backup date */
    sizeof(revisions),ATR$C_ASCDATES,&revisions, /* number of revisions */
    sizeof(prot),ATR$C_FPRO,&prot,         /* file protection  */
    sizeof(uic),ATR$C_UIC,&uic,            /* file owner */
    sizeof(jnl),ATR$C_JOURNAL,&jnl,        /* journal flags */
    0,0,0
  } ;
#endif 
  static char EName[NAM$C_MAXRSS];
  static char RName[NAM$C_MAXRSS];
  static struct dsc$descriptor_s FileName =
    {0,DSC$K_DTYPE_T,DSC$K_CLASS_S,0};
  static struct dsc$descriptor_s string = {0,DSC$K_DTYPE_T,DSC$K_CLASS_S,0};
  static short int DevChan;
  static short int iosb[4];
  static long int i,j,status;
  static int fibp;
 
    /* initialize RMS structures, we need a NAM to retrieve the FID */
    Fab = cc$rms_fab;
    Fab.fab$l_fna = file; /* name of file */
    Fab.fab$b_fns = strlen(file);
    Fab.fab$l_nam = &Nam; /* FAB has an associated NAM */
    Nam = cc$rms_nam;
    Nam.nam$l_esa = (char *)&EName; /* expanded filename */
    Nam.nam$b_ess = sizeof(EName);
    Nam.nam$l_rsa = (char *)&RName; /* resultant filename */
    Nam.nam$b_rss = sizeof(RName);
 
    /* do $PARSE and $SEARCH here */
    status = sys$parse(&Fab);
    if (!(status & 1)) return(status);
 
    /* search for the first file.. If none signal error */
    status = sys$search(&Fab);
    if (!(status & 1)) return(status);
 
      /* initialize Device name length, note that this points into the NAM
         to get the device name filled in by the $PARSE, $SEARCH services */
      DevDesc.dsc$w_length = Nam.nam$t_dvi[0];
 
      status = sys$assign(&DevDesc,&DevChan,0,0);
      if (!(status & 1)) return(status);
 
      FileName.dsc$a_pointer = Nam.nam$l_name;
      FileName.dsc$w_length = Nam.nam$b_name+Nam.nam$b_type+Nam.nam$b_ver;
 
      /* Initialize the FIB */
      for (i=0;i<3;i++)
        Fib.fib$r_fid_overlay.fib$w_fid[i]=Nam.nam$w_fid[i];
      for (i=0;i<3;i++)
        Fib.fib$r_did_overlay.fib$w_did[i]=Nam.nam$w_did[i];
 
      /* Use the IO$_ACCESS function to return info about the file */
      /* Note, used this way, the file is not opened, and the expiration */
      /* and revision dates are not modified */
      status = sys$qiow(0,DevChan,IO$_ACCESS,&iosb,0,0,
                        &FibDesc,&FileName,0,0,&Atr,0);
      if (!(status & 1)) return(status);
      status = iosb[0];
      if (!(status & 1)) return(status);

      unix_to_vmstime(utimebuf->actime,&Cdate);
      unix_to_vmstime(utimebuf->modtime,&Rdate);

      /* note, part of the FIB was cleared by earlier QIOW, so reset it */
      Fib.fib$r_acctl_overlay.fib$l_acctl = FIB$M_NORECORD;
      for (i=0;i<3;i++)
        Fib.fib$r_fid_overlay.fib$w_fid[i]=Nam.nam$w_fid[i];
      for (i=0;i<3;i++)
        Fib.fib$r_did_overlay.fib$w_did[i]=Nam.nam$w_did[i];
 
      /* Use the IO$_MODIFY function to change info about the file */
      /* Note, used this way, the file is not opened, however this would */
      /* normally cause the expiration and revision dates to be modified. */
      /* Using FIB$M_NORECORD prohibits this from happening. */
      status = sys$qiow(0,DevChan,IO$_MODIFY,&iosb,0,0,
                        &FibDesc,&FileName,0,0,&Atr,0);
      if (!(status & 1)) return(status);
 
      status = iosb[0];
      if (!(status & 1)) return(status);
 
      status = sys$dassgn(DevChan);
      if (!(status & 1)) return(status);
      return(1); 
}
 

/*
 * UNIX_TO_VMSTIME().  Steve Drew 1990.
 *
 * Input: longword C time format.
 * Output: Vaxtime quadword
 *
 */
unix_to_vmstime(time_l,vaxtime_val)
time_t time_l;
VAXTIME_PTR(vaxtime_val);
{
    static char tmpdate[30];
    struct tm *tm;
    char months[12][4] = { "JAN","FEB","MAR","APR","MAY","JUN",
                           "JUL","AUG","SEP","OCT","NOV","DEC" }; 

    tm = localtime(&time_l);
    sprintf(tmpdate,"%02d-%s-19%d %02d:%02d:%02d.00",
        tm->tm_mday,months[tm->tm_mon],tm->tm_year,tm->tm_hour,
        tm->tm_min,tm->tm_sec);    
    return(bin_date(tmpdate,vaxtime_val));
}

/*
 * BIN_DATE
 *
 * Input: DD-MMM-YYYY HH:MM:SS.CC
 * Output: Vaxtime quad word
 * if vaxtime_s is NULL then set to current date/time (zero tenths seconds)
 */
#pragma nostandard
bin_date(vaxtime_s,vaxtime_val)
char *vaxtime_s;
VAXTIME_PTR(vaxtime_val);
{
    $DESCRIPTOR(_vaxtime_s,vaxtime_s);
    $DESCRIPTOR(_now,"-- ::.00");

    /*
     * return current time
     */
    if (vaxtime_s == NULL)  {
        sys$bintim(&_now,vaxtime_val);
        return(1);
    }

    /*
     * convert given date to binary
     */
    _vaxtime_s.dsc$w_length = strlen(vaxtime_s);
    if (sys$bintim(&_vaxtime_s,vaxtime_val) == SS$_NORMAL) {
        return(1);
    }
    return(0);
}
#pragma standard
